#
# NDK build file
#
# Written by Ant-ON, 2021-2022
#

#
# Base on cryptsetup 2.6.0
# https://gitlab.com/cryptsetup/
#

LOCAL_PATH := $(call my-dir)
include $(CLEAR_VARS)

# After changing the module name, needs to correct the name in the bm_module.h 
LOCAL_MODULE := cryptsetup-2.6

LOCAL_C_INCLUDES := $(LOCAL_PATH) \
	$(LOCAL_PATH)/../include \
	$(LOCAL_PATH)/include \
	$(LOCAL_PATH)/src \
	$(LOCAL_PATH)/src/luks1 \
	$(LOCAL_PATH)/src/luks2 \
	$(LOCAL_PATH)/src/crypto_backend \
	$(LOCAL_PATH)/src/integrity \
	$(LOCAL_PATH)/src/loopaes \
	$(LOCAL_PATH)/src/tcrypt \
	$(LOCAL_PATH)/src/verity \
	$(LOCAL_PATH)/src/bitlk

LOCAL_SRC_FILES := \
	bdevice.c \
	cryptsetup-java.c \
	libdevmapper.c \
	uuid.c \
	crc32_with_hw.c \
	utils.c \
	crypto_mbedtls.c \
	cipher_check_mbedtls.c \
	filter_elephant.c \
	src/setup.c \
	src/utils.c \
	src/utils_benchmark.c \
	src/utils_crypt.c \
	src/utils_keyring.c \
	src/utils_pbkdf.c \
	src/utils_safe_memory.c \
	src/volumekey.c \
	src/random.c \
	src/crypt_plain.c \
	src/keyslot_context.c \
	src/bitlk/bitlk.c \
	src/luks1/af.c \
	src/luks1/keyencryption.c \
	src/luks1/keymanage.c \
	src/luks2/luks2_disk_metadata.c \
	src/luks2/luks2_json_format.c \
	src/luks2/luks2_json_metadata.c \
	src/luks2/luks2_digest.c \
	src/luks2/luks2_digest_pbkdf2.c \
	src/luks2/luks2_keyslot.c \
	src/luks2/luks2_keyslot_luks2.c \
	src/luks2/luks2_segment.c \
	src/luks2/luks2_token_keyring.c \
	src/luks2/luks2_token.c \
	src/luks2/luks2_reencrypt.c \
	src/luks2/luks2_reencrypt_digest.c \
	src/luks2/luks2_keyslot_reenc.c \
	src/luks2/luks2_luks1_convert.c \
	src/tcrypt/tcrypt.c \
	src/integrity/integrity.c \
	src/verity/rs_decode_char.c \
	src/verity/rs_encode_char.c \
	src/verity/verity.c \
	src/verity/verity_fec.c \
	src/verity/verity_hash.c \
	src/loopaes/loopaes.c \
	src/fvault2/fvault2.c \
	src/utils_blkid.c \
	src/crypto_backend/base64.c \
	src/crypto_backend/crypto_cipher_kernel.c \
	src/crypto_backend/crypto_storage.c \
	src/crypto_backend/pbkdf_check.c \
	src/crypto_backend/utf8.c \
	src/crypto_backend/argon2_generic.c \
	src/crypto_backend/cipher_generic.c \
	src/crypto_backend/argon2/blake2/blake2b.c \
	src/crypto_backend/argon2/argon2.c \
	src/crypto_backend/argon2/core.c \
	src/crypto_backend/argon2/encoding.c \
	src/crypto_backend/argon2/ref.c

MAIN_CFLAGS :=  \
	--include='config.h' \
	-DEXTERNAL_LUKS2_TOKENS_PATH=NULL \
	-DARGON2_NO_THREADS -DAND_MAX_FD=16 \
	-DANDROID_JNI -DTCRYPT_MBEDTLS -DUSE_BM_BACKEND

LOCAL_CFLAGS += $(MAIN_CFLAGS) -DLOGTAG=\"cryptsetup\" -O2 \
	\
	-fno-strict-aliasing \
	\
	-Wall

# LTO
LOCAL_CFLAGS += -fuse-linker-plugin -flto
LOCAL_LDFLAGS += -fuse-linker-plugin -flto

# Remove unused code
LOCAL_CFLAGS += -fvisibility=hidden -ffunction-sections -fdata-sections -Wl,--gc-sections
LOCAL_CPPFLAGS += -fvisibility=hidden -fvisibility-inlines-hidden -ffunction-sections -fdata-sections -Wl,--gc-sections
LOCAL_LDFLAGS += -Wl,--gc-sections -Wl,-z,nocopyreloc

# Disable STB_GNU_UNIQUE symbol binding
LOCAL_CFLAGS += -Wl,--no-gnu-unique
LOCAL_LDFLAGS += -Wl,--no-gnu-unique

# Use gold
#LOCAL_LDFLAGS += -fuse-ld=gold

LOCAL_SHARED_LIBRARIES := mbedtls-2.25.0 bm-1.4
LOCAL_STATIC_LIBRARIES := libjson-0.12

LOCAL_EXPORT_C_INCLUDES := $(LOCAL_PATH)

LOCAL_LDLIBS += -llog

include $(BUILD_SHARED_LIBRARY)
